@@ -55,7 +55,7 @@ DEFAULT_URL = "https://portal.yellowdog.co/api"
 @dataclass
 class ConfigWorkRequirement:
     args: List[str] = field(default_factory=list)
-    bash_script: Optional[str] = None
+    bash_script: Optional[str] = None  # Deprecated
     capture_taskoutput: bool = True
     completed_task_ttl: Optional[float] = None  # In minutes
     csv_files: Optional[List[str]] = None
@@ -104,7 +104,7 @@ CR_BATCH_SIZE = 2000
 
 @dataclass
 class ConfigWorkerPool:
-    auto_scaling_idle_delay: float = 10
+    auto_scaling_idle_delay: float = 10  # Deprecated
     auto_shutdown: bool = True
     auto_shutdown_delay: float = 10
     compute_requirement_batch_size: int = CR_BATCH_SIZE
@@ -115,9 +115,12 @@ class ConfigWorkerPool:
     min_nodes: int = 0
     name: Optional[str] = None
     node_boot_time_limit: float = 10
+    node_idle_grace_period: float = 2
+    node_idle_time_limit: float = 5
     target_instance_count: int = 0
     template_id: Optional[str] = None
     user_data: Optional[str] = None
+    user_data_file: Optional[str] = None
     worker_pool_data_file: Optional[str] = None
     worker_tag: Optional[str] = None
     workers_per_vcpu: Optional[int] = None
@@ -199,9 +202,16 @@ def load_config_common() -> ConfigCommon:
         url = substitute_mustache_str(common_section.get(URL, DEFAULT_URL))
         if url != DEFAULT_URL:
             print_log(f"Using the YellowDog API at: {url}")
+
+        # Exhaustive Mustache processing for common section variables
+        # Note that add_substitutions() will perform all possible
+        # substitutions for the items in its dictionary each time it's
+        # called
+        add_substitutions(subs={URL: url})
         namespace = substitute_mustache_str(common_section[NAMESPACE])
+        add_substitutions(subs={NAMESPACE: namespace})
         name_tag = substitute_mustache_str(common_section[NAME_TAG])
-        add_substitutions(subs={NAMESPACE: namespace, NAME_TAG: name_tag, URL: url})
+        add_substitutions(subs={NAME_TAG: name_tag})
 
         return ConfigCommon(
             # Required
@@ -375,9 +385,14 @@ def load_config_worker_pool() -> Optional[ConfigWorkerPool]:
                 wp_section.get(WP_NAME, None),
             ),
             node_boot_time_limit=wp_section.get(NODE_BOOT_TIME_LIMIT, 10),
+            node_idle_grace_period=wp_section.get(NODE_IDLE_GRACE_PERIOD, 2),
+            node_idle_time_limit=wp_section.get(
+                NODE_IDLE_TIME_LIMIT, wp_section.get(AUTO_SCALING_IDLE_DELAY, 5)
+            ),
             target_instance_count=wp_section.get(TARGET_INSTANCE_COUNT, 1),
             template_id=wp_section.get(TEMPLATE_ID, None),
             user_data=wp_section.get(USERDATA, None),
+            user_data_file=wp_section.get(USERDATAFILE, None),
             worker_pool_data_file=worker_pool_data_file,
             worker_tag=worker_tag,
             workers_per_vcpu=wp_section.get(WORKERS_PER_VCPU, None),
@@ -446,3 +461,13 @@ entities = {
     WorkRequirement: "work",
     ComputeRequirement: "compute",
 }
+
+
+def update_config_work_requirement(config_wr: ConfigWorkRequirement):
+    """
+    Update a ConfigWorkRequirement Object with the current dictionary of
+    Mustache substitutions. Returns the updated object.
+    """
+    config_wr_str_processed = substitute_mustache_str(str(config_wr))
+    # Note: 'literal_eval' doesn't work here
+    return eval(config_wr_str_processed)
